/*****************************************************************************
*
* File               i2c_2dig_7seg.c
* Date               1/26/10
* Processor          ATtiny45
*
* Description        Display digits and error codes (E0, E1, etc) on a pair
* of LED seven segment displays via a 16 bit I2C I/O expander
*
****************************************************************************/

#include <avr/io.h>
#include <util/delay.h>

#include "defines.h"
#include "i2c_2dig_7seg.h"
#include "USI_TWI_Master.h"


//IO expander bit definitions, specific to device and how it's wired up
const byte RIGHT_B_SEVENSEG[] = {
   0xB0,
   0x80,
   0x30,
   0xA0,
   0x80,
   0xA0,
   0xB0,
   0x80,
   0xB0,
   0xA0,
   0xB0,
   0xB0,
   0x30,
   0xB0,
   0x30,
   0x10
};

const byte RIGHT_A_SEVENSEG[] = {
   0x07,
   0x02,
   0x0B,
   0x0B,
   0x0E,
   0x0D,
   0x0D,
   0x03,
   0x0F,
   0x0F,
   0x08,
   0x0C,
   0x08,
   0x0A,
   0x0D,
   0x0D
};

const byte LEFT_B_SEVENSEG[] = {
   0x07,
   0x04,
   0x03,
   0x05,
   0x04,
   0x05,
   0x07,
   0x04,
   0x07,
   0x05,
   0x07,
   0x07,
   0x03,
   0x07,
   0x03,
   0x02
};

const byte LEFT_A_SEVENSEG[] = {
   0xB0,
   0x10,
   0x70,
   0x70,
   0xD0,
   0xE0,
   0xE0,
   0x30,
   0xF0,
   0xF0,
   0x40,
   0xC0,
   0x40,
   0x50,
   0xE0,
   0xE0
};

#define DOTS_A_SEVENSEG 0x00
#define DOTS_B_SEVENSEG 0x48

#define DASH_A_SEVENSEG 0x48
#define DASH_B_SEVENSEG 0x00

byte messageBuf[MESSAGEBUF_SIZE];

byte blink_count = 0;

byte on = 0;

void led_init(){
	byte i;

   USI_TWI_Master_Initialise();

   messageBuf[0] = IOEXP_ADDR<<1;

   //IOCON, sequential banks, no slew control, INT open drain
   messageBuf[1] = IOCON;
   messageBuf[2] = DISSLW | ODR;
   if (!USI_TWI_Start_Read_Write( messageBuf, 3)){
   	for(i=0;i<1;i++){
			B_HIGH;
			_delay_ms(1000);
			B_LOW;
			_delay_ms(1000);
   	}
   	_delay_ms(4000);
   }

   //IODIR, all outputs
   messageBuf[1] = IODIR;
   messageBuf[2] = 0;
   messageBuf[3] = 0;
   if (!USI_TWI_Start_Read_Write( messageBuf, 4)){
#if 1
   	for(i=0;i<2;i++){
			B_HIGH;
			_delay_ms(1000);
			B_LOW;
			_delay_ms(1000);
   	}
   	_delay_ms(4000);
#endif
   }
}//void led_init()

/* Display the number sent on the seven segment display
 * if the number is below 0 or above 99 make the display
 * blink the number
 */
void led_dec(int val){
   byte left, right,i;
   byte blink;

   if(val < 0 ){
   	blink = TRUE;
   	val *= -1;
   }else if (val > 100){
   	blink = TRUE;
   	val -= 100;
   }else{
   	blink = FALSE;
   }

   messageBuf[0] = IOEXP_ADDR<<1;
   left = val/10;
   right = val - (left * 10);

   messageBuf[1] = GPIOA;
#if 0
   if(val>99){	//out of range, display 2 dashes
	   messageBuf[2] = DASH_A_SEVENSEG;
   }else{
	   messageBuf[2] = RIGHT_A_SEVENSEG[right] | LEFT_A_SEVENSEG[left];
   }
#endif
   if(blink){
   	_delay_ms(BLINK_DELAY);
		if(on==1){
			//on = 0;
			messageBuf[2] = RIGHT_A_SEVENSEG[right] | LEFT_A_SEVENSEG[left];
		}else{
			//on = 1;
			//right = 0;
			//left = 0;
			messageBuf[2] = 0;//RIGHT_A_SEVENSEG[right] | LEFT_A_SEVENSEG[left];
		}
   }else{
   	messageBuf[2] = RIGHT_A_SEVENSEG[right] | LEFT_A_SEVENSEG[left];
   }
   if(!USI_TWI_Start_Read_Write( messageBuf, 3)){

   	for(i=0;i<3;i++){
			B_HIGH;
			_delay_ms(1000);
			B_LOW;
			_delay_ms(1000);
   	}
   	_delay_ms(4000);

   }


   messageBuf[1] = GPIOB;
#if 0
   if(val>99){	//out of range, display 2 dashes
	   messageBuf[2] = DASH_B_SEVENSEG;
   }else{
	   messageBuf[2] = RIGHT_B_SEVENSEG[right] | LEFT_B_SEVENSEG[left];
   }
#endif
   if(blink){
   	//_delay_ms(BLINK_DELAY);
		if(on==1){
			on = 0;
			messageBuf[2] = RIGHT_B_SEVENSEG[right] | LEFT_B_SEVENSEG[left];
		}else{
			on = 1;
			//right = 0;
			//left = 0;
			messageBuf[2] = 0;//RIGHT_B_SEVENSEG[right] | LEFT_B_SEVENSEG[left];
		}
   }else{
   	messageBuf[2] = RIGHT_B_SEVENSEG[right] | LEFT_B_SEVENSEG[left];
   }
   if(!USI_TWI_Start_Read_Write( messageBuf, 3)){

   	for(i=0;i<4;i++){
			B_HIGH;
			_delay_ms(1000);
			B_LOW;
			_delay_ms(1000);
   	}
   	_delay_ms(4000);

	}

}


void led_hex(byte val){
   byte left, right,i;
   messageBuf[0] = IOEXP_ADDR<<1;
   messageBuf[1] = GPIO;
   left = val >> 4;
   right = val - (left << 4);
   messageBuf[2] = RIGHT_A_SEVENSEG[right] | LEFT_A_SEVENSEG[left];
   messageBuf[3] = RIGHT_B_SEVENSEG[right] | LEFT_B_SEVENSEG[left] | DOTS_B_SEVENSEG;
   if(!USI_TWI_Start_Read_Write( messageBuf, 4)){

   	for(i=0;i<5;i++){
			B_HIGH;
			_delay_ms(1000);
			B_LOW;
			_delay_ms(1000);
   	}
   	_delay_ms(4000);

   }

}
