#define byte unsigned char

#include <avr/io.h>
#include <util/delay.h>

#include "defines.h"
#include "temponewire.h"

void CRC8(byte input, byte *seed);
void write_bit(byte bit);
byte read_bit(void);
void write_byte(byte data);
byte read_byte(void);
byte reset(void);

int get_temp(){

   byte temp_lsb;
   byte temp_msb;
   int celc;
   int farn;
   byte remainder;

   if(reset()){
		B_HIGH;
		_delay_ms(1000);
		B_LOW;
		_delay_ms(3000);
   }

   write_byte(SKIP_ROM);
   write_byte(CONVERT_T);
   while(!read_bit());  //can read bits (time slots) which will be 0 until conversion complete
   if(reset()){
   	B_HIGH;
   	_delay_ms(2000);
   	B_LOW;
   	_delay_ms(3000);
   }
   write_byte(SKIP_ROM);
   write_byte(READ_SCRATCHPAD);
   temp_lsb = read_byte();
   temp_msb = read_byte();
   //celc = temp_msb<<4;
   //celc += temp_lsb>>4;
   celc = temp_msb<<8 | temp_lsb;
  // if(temp_lsb&8) //round up .5 or more
    //  celc+=1;

   //farn = celc;
   farn = (celc>>4)*9/5+32;
   remainder = celc & 0xF;
   if(remainder <= 4){
   	//do nothing
   }else if(remainder <= 13){
   	farn += 1;
   }else{
   	farn += 2;
   }

   return(farn);
}

void init_temp(){
   if(reset()){
   	B_HIGH;
   	_delay_ms(3000);
   	B_LOW;
   	_delay_ms(3000);
   }
   write_byte(SKIP_ROM);
   write_byte(WRITE_SCRATCHPAD);
   write_byte(0x12);
   write_byte(0x34);
   write_byte(0xEF);
}


void write_bit(byte bit){
   CONFIGTOUTPUT;
   T_LOW;
   if(bit)
   {
      _delay_us(6);      // 10.58us
      CONFIGTINPUT;
      _delay_us(64);     // 61.82us
   }
   else
   {
      _delay_us(60);     // 61.82us
      CONFIGTINPUT;             // better releash the bus
      _delay_us(10);      // 10.58us
   }
}


byte read_bit(void)
{
   byte data;

   CONFIGTOUTPUT;
   T_LOW;
   //_delay_us(0);         // 10.58us  <--This looks like a mistake in the original code
   _delay_us(6);         // 10.58us  <--What it should be
   CONFIGTINPUT;
   //_delay_us(0);         // 10.58us  <--This looks like a mistake in the original code
   _delay_us(9);         // 10.58us  <--What it should be
   data=T_IN;
   _delay_us(55);        // 61.82us

   return data;
}


void write_byte(byte data)
{
    byte i;

    for(i=0; i<8; i++)
    {
        write_bit(data & 0x01);
        data>>=1;
    }
}



byte read_byte(void)
{
    byte i, bit, data;
    data=0;
    for(i=0; i<8; i++)
    {
        bit=read_bit();
        if(bit)
         data|=_BV(i);
    }

    return data;
}


byte reset(void)
{
    byte present;

    CONFIGTOUTPUT;
    T_LOW;
    _delay_us(481);  // 481.10us
    CONFIGTINPUT;
    _delay_us(70);                 // 61.82us
    present=T_IN;
    _delay_us(410);  // 410.34us

    // 0 for device present
    return present;
}

